# -*- coding: UTF-8 -*-
# @Time : 2021/11/27 下午6:15 
# @Author : 刘洪波
import json


class Producer(object):
    def __init__(self, client, topic, schema):
        self.producer = client.create_producer(topic, schema=schema)

    def send(self, msg, _async=True, callback=None, random_topic=None):
        """
        发送消息
        :param msg: 需要发送的消息
        :param _async: 是否异步发送消息， True异步发送， Flase 同步发送
        :param callback: 异步发送时的回调函数
        :param random_topic: 是否有 随机topic
        :return:
        """
        if _async:
            if msg:
                if isinstance(msg, list):
                    if random_topic:
                        for m in msg:
                            if m:
                                m = dict(json.loads(m))
                                m['random_topic'] = random_topic
                                m = json.dumps(m)
                                self.producer.send_async(m, callback=callback)
                    else:
                        for m in msg:
                            if m:
                                self.producer.send_async(m, callback=callback)
                elif isinstance(msg, str):
                    if random_topic:
                        msg = dict(json.loads(msg))
                        msg['random_topic'] = random_topic
                        msg = json.dumps(msg)
                    self.producer.send_async(msg, callback=callback)
                else:
                    raise ValueError('the data type sent is wrong, the required type is list or str')
            else:
                raise ValueError('No data to send, the data is Empty')
        else:
            if msg:
                if isinstance(msg, list):
                    if random_topic:
                        for m in msg:
                            if m:
                                m = dict(json.loads(m))
                                m['random_topic'] = random_topic
                                m = json.dumps(m)
                                self.producer.send(m)
                    else:
                        for m in msg:
                            if m:
                                self.producer.send(m)
                elif isinstance(msg, str):
                    if random_topic:
                        msg = dict(json.loads(msg))
                        msg['random_topic'] = random_topic
                        msg = json.dumps(msg)
                    self.producer.send(msg)
                else:
                    raise ValueError('the data type sent is wrong, the required type is list or str')
            else:
                raise ValueError('No data to send, the data is Empty')

    def close(self):
        self.producer.close()



# -*- coding: UTF-8 -*-
# @Time : 2021/12/1 下午4:04 
# @Author : 刘洪波
import pulsar
import json
import time
from pulsar_mq.Producers import Producer
from pulsar_mq.Consumers import Consumer

"""
pulsar服务
一、 ConsumeProduce
1. 订阅pulsar
2. 处理消费的数据
3. 发送业务数据
4. 发送的 队列 可以是 随机的

二、 ProduceConsume

1. 发送数据至pulsar
2. 订阅pulsar得到业务数据
3. 消费的 队列 可以是 随机的
"""


class ConsumeProduce(object):
    """订阅得到业务数据，经处理过后发送"""

    def __init__(self, client, consumer_topic, consumer_name: str, producer_topic=None,
                 schema=pulsar.schema.StringSchema()):
        """
        创建 生产者 和 消费者
        :param client:
        :param consumer_topic:
        :param consumer_name:
        :param producer_topic:
        :param schema:
        """
        self.client = client
        self.sch = schema
        self.consumer = Consumer(client, consumer_topic, consumer_name, schema=schema)
        if producer_topic:
            self.producer = Producer(client, producer_topic, schema=schema)
        else:
            self.producer = None

    def run(self, task, thread_count=None, _async=True, callback=None, logger=None):
        """
        :param task: 任务程序
        :param thread_count: 指定最大线程数
        :param _async: 是否异步发送消息， True异步发送， Flase 同步发送
        :param callback: 异步发送的回调函数
        :param logger: 日志收集器
        :return:
        """

        def send_task(msg):
            result = task(msg)
            data = json.loads(msg.data())
            random_topic = data.get('random_topic')
            if random_topic:
                producer = Producer(self.client, random_topic, schema=self.sch)
                producer.send(result, _async=_async, callback=callback)
                producer.close()
            elif self.producer:
                self.producer.send(result, _async=_async, callback=callback)
            else:
                raise ValueError('producer_topic and random_topic is None, need producer topic')

        self.consumer.receive(send_task, thread_count, logger)


class ProduceConsume(object):
    """发送数据至pulsar，订阅pulsar得到业务数据"""

    def __init__(self, client, producer_topic, consumer_topic=None, consumer_name=None,
                 schema=pulsar.schema.StringSchema()):
        self.client = client
        self.sch = schema
        self.consumer_topic = consumer_topic
        self.consumer_name = consumer_name
        self.producer = Producer(client, producer_topic, schema=schema)

    def run(self, data, random_topic=True, timeout_millis=300000, _async=True, callback=None):
        """
        :param data: 输入格式为json
        :param random_topic: 是否随机队列
        :param timeout_millis: 订阅超时
        :param _async: 是否异步发送消息， True异步发送， Flase 同步发送
        :param callback: 异步发送的回调函数
        :return:
        """
        if random_topic:
            random_topic = 'cdss_random_topic_' + str(int(round(time.time() * 1000000)))
            self.consumer_topic = random_topic
            self.consumer_name = random_topic
        if self.consumer_topic and self.consumer_name:
            consumer = self.client.subscribe(self.consumer_topic, self.consumer_name, schema=self.sch)
            self.producer.send(data, _async=_async, callback=callback, random_topic=random_topic)
            msgs = []
            if isinstance(data, list):
                for i in range(len(data)):
                    msg = consumer.receive(timeout_millis)
                    consumer.acknowledge(msg)
                    msgs.append(msg)
                consumer.close()
                return msgs
            elif isinstance(data, str):
                msg = consumer.receive(timeout_millis)
                consumer.acknowledge(msg)
                consumer.close()
                return msg
            else:
                raise ValueError('the data type sent is wrong, the required type is list or str')
        else:
            raise ValueError('consumer_topic and random_topic is None, need consumer topic')



# -*- coding: UTF-8 -*-
# @Time : 2021/12/2 下午5:55 
# @Author : 刘洪波
import pika
from concurrent.futures import ThreadPoolExecutor


class Connection(object):
    def __init__(self, host, port, username, password):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.queue_name = None

    def create_queue(self, exchange, routing_key, durable=False):
        """
        创建 消息队列
        :param exchange:
        :param routing_key:
        :param durable:
        :return:
        """
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=self.host, port=self.port, credentials=pika.PlainCredentials(self.username, self.password))
        )
        channel = connection.channel()
        channel.exchange_declare(exchange=exchange, exchange_type='topic', durable=durable)
        result = channel.queue_declare(queue='')
        self.queue_name = result.method.queue
        channel.queue_bind(exchange=exchange, queue=self.queue_name, routing_key=routing_key)
        connection.close()

    def consumer(self, task):
        """
        消费者
        :param task:  处理消息的业务功能
        :return:
        """
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=self.host, port=self.port, credentials=pika.PlainCredentials(self.username, self.password))
        )
        channel = connection.channel()

        def callback(ch, method, properties, body):
            body = body.decode('utf8')
            task(body)
            ch.basic_ack(delivery_tag=method.delivery_tag)
        channel.basic_consume(queue=self.queue_name, auto_ack=False, on_message_callback=callback)
        channel.basic_qos(prefetch_count=1)
        channel.start_consuming()

    def send(self, message_list: list, exchange, routing_key, durable=False):
        """
        发送数据
        :param message_list:
        :param exchange:
        :param routing_key:
        :param durable:
        :return:
        """
        connection = pika.BlockingConnection(pika.ConnectionParameters(
            host=self.host, port=self.port, credentials=pika.PlainCredentials(self.username, self.password))
        )
        channel = connection.channel()
        channel.exchange_declare(exchange=exchange, exchange_type='topic', durable=durable)
        for message in message_list:
            channel.basic_publish(exchange=exchange, routing_key=routing_key, body=message)
        connection.close()

    def receive(self, exchange, routing_key, task, durable=False, thread_count=None):
        """
        消费者消费
        :param exchange:
        :param routing_key:
        :param task:
        :param durable:
        :param thread_count: 当thread_count 有值的时候，可以进行多线程并行消费
        :return:
        """
        self.create_queue(exchange, routing_key, durable)
        if thread_count:
            pool = ThreadPoolExecutor(max_workers=thread_count)
            for i in range(thread_count):
                pool.submit(self.consumer, task)
        else:
            self.consumer(task)

    def service(self, task, consumer_exchange, consumer_routing_key,
                producer_exchange, producer_routing_key, durable=False, thread_count=None):
        """
        rabbitmq 服务
        1. 订阅rabbitmq
        2. 处理消费的数据
        3. 发送得到的结果
        :param task:
        :param consumer_exchange:
        :param consumer_routing_key:
        :param producer_exchange:
        :param producer_routing_key:
        :param thread_count:
        :param durable:
        :return:
        """
        def callback(body):
            result = task(body)
            if result:
                self.send(result, producer_exchange, producer_routing_key, durable)
        self.receive(consumer_exchange, consumer_routing_key, callback, durable, thread_count)




# -*- coding: UTF-8 -*-
# @Time : 2021/11/27 下午5:09 
# @Author : 刘洪波

import pulsar


def client(url: str):
    from pulsar_mq.Clients import Client
    return Client(url)


def create_consumer(clients, topic, consumer_name: str, schema=pulsar.schema.StringSchema()):
    from pulsar_mq.Consumers import Consumer
    return Consumer(clients, topic, consumer_name, schema)


def create_producer(clients, topic, schema):
    from pulsar_mq.Producers import Producer
    return Producer(clients, topic, schema)



# -*- coding: UTF-8 -*-
# @Time : 2021/11/27 下午5:36 
# @Author : 刘洪波

from concurrent.futures import ThreadPoolExecutor


class Consumer(object):
    def __init__(self, client, topic, consumer_name, schema):
        self.consumer = client.subscribe(topic, consumer_name, schema=schema)

    def receive(self, task, thread_count=None, logger=None):
        """
        :param task: 任务程序
        :param thread_count: 指定最大线程数
        :param logger: 日志收集器
        :return:
        """
        if thread_count:
            """多线程处理"""
            pool = ThreadPoolExecutor(max_workers=thread_count)
            while True:
                msg = self.consumer.receive()
                self.consumer.acknowledge(msg)
                pool.submit(self._acknowledge, task, msg, logger)
        else:
            """消费一个，处理一个"""
            while True:
                msg = self.consumer.receive()
                self.consumer.acknowledge(msg)
                self._acknowledge(task, msg, logger)

    @staticmethod
    def _acknowledge(task, msg, logger):
        try:
            task(msg)
        except Exception as e:
            # 消息未被成功处理
            if logger:
                logger.error('消费异常，报错信息如下：')
                logger.error(e)
            else:
                print('消费异常，报错信息如下：')
                print(e)

    def close(self):
        self.consumer.close()



# -*- coding: UTF-8 -*-
# @Time : 2021/12/6 上午11:33 
# @Author : 刘洪波
import setuptools
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='mq-fw',
    version='1.3.0',
    packages=setuptools.find_packages(),
    url='https://gitee.com/maxbanana',
    license='Apache',
    author='hongbo liu',
    author_email='782027465@qq.com',
    description='A message queue framework, including pulsar and rabbitmq',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['pulsar-client', 'pika'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


# -*- coding: UTF-8 -*-
# @Time : 2021/12/2 下午5:54 
# @Author : 刘洪波


def connect(host, port, username, password):
    from rabbitmq.Connections import Connection
    return Connection(host, port, username, password)
